import { useEffect, useState } from "react";
import { Link } from "react-router-dom";

interface Feed {
  id: string;
  url: string;
  title?: string;
  description?: string;
  category?: string;
  lastUpdated?: string;
  createdAt: string;
  active: boolean;
}

function FeedList() {
  const [feeds, setFeeds] = useState<Feed[]>([]);
  const [filteredFeeds, setFilteredFeeds] = useState<Feed[]>([]);
  const [categories, setCategories] = useState<string[]>([]);
  const [selectedCategory, setSelectedCategory] = useState<string>("");
  const [loading, setLoading] = useState(true);
  const [error, setError] = useState<string | null>(null);
  
  // Pagination state
  const [currentPage, setCurrentPage] = useState(1);
  const [pageSize, setPageSize] = useState(12);
  const [totalPages, setTotalPages] = useState(1);
  const [totalFeeds, setTotalFeeds] = useState(0);

  useEffect(() => {
    fetchFeeds();
    fetchCategories();
  }, []);

  useEffect(() => {
    // Reset to page 1 when category changes
    setCurrentPage(1);
    fetchFeeds();
  }, [selectedCategory]);

  useEffect(() => {
    // Fetch feeds when page changes
    fetchFeeds();
  }, [currentPage, pageSize]);

  const fetchFeeds = async () => {
    try {
      setLoading(true);
      setError(null);
      
      // Build query parameters
      const params = new URLSearchParams();
      params.append("page", currentPage.toString());
      params.append("limit", pageSize.toString());
      if (selectedCategory) {
        params.append("category", selectedCategory);
      }
      
      const response = await fetch(`/api/feeds?${params.toString()}`);
      if (!response.ok) {
        const errorData = await response.json();
        throw new Error(errorData.error || "フィードの取得に失敗しました");
      }
      
      const data = await response.json();
      
      // Handle paginated response
      if (data.feeds && typeof data.total !== 'undefined') {
        setFeeds(data.feeds);
        setFilteredFeeds(data.feeds);
        setTotalFeeds(data.total);
        setTotalPages(data.totalPages);
      } else {
        // Fallback for non-paginated response (backward compatibility)
        setFeeds(data.feeds || []);
        setFilteredFeeds(data.feeds || []);
        setTotalFeeds(data.feeds?.length || 0);
        setTotalPages(1);
      }
    } catch (err) {
      console.error("Feed fetch error:", err);
      setError(err instanceof Error ? err.message : "エラーが発生しました");
    } finally {
      setLoading(false);
    }
  };

  const fetchCategories = async () => {
    try {
      const response = await fetch("/api/categories");
      if (response.ok) {
        const data = await response.json();
        setCategories(data.categories || []);
      }
    } catch (err) {
      console.error("Error fetching categories:", err);
    }
  };


  const formatDate = (dateString: string) => {
    return new Date(dateString).toLocaleString("ja-JP");
  };

  if (loading) {
    return <div className="loading">読み込み中...</div>;
  }

  if (error) {
    return <div className="error">{error}</div>;
  }

  if (filteredFeeds.length === 0 && feeds.length > 0 && selectedCategory) {
    return (
      <div className="empty-state">
        <p>カテゴリ「{selectedCategory}」のフィードがありません</p>
        <button
          className="btn btn-secondary"
          onClick={() => setSelectedCategory("")}
          style={{ marginTop: "10px" }}
        >
          全てのフィードを表示
        </button>
      </div>
    );
  }

  if (feeds.length === 0) {
    return (
      <div className="empty-state">
        <p>アクティブなフィードがありません</p>
        <p>
          フィードリクエストでRSSフィードをリクエストするか、管理者にフィード追加を依頼してください
        </p>
        <button
          className="btn btn-secondary"
          onClick={fetchFeeds}
          style={{ marginTop: "10px" }}
        >
          再読み込み
        </button>
      </div>
    );
  }

  return (
    <div>
      <div
        style={{
          marginBottom: "20px",
          display: "flex",
          justifyContent: "space-between",
          alignItems: "center",
        }}
      >
        <h2>
          フィード一覧 (
          {selectedCategory
            ? `${totalFeeds}件 - カテゴリ: ${selectedCategory}`
            : `${totalFeeds}件`}
          )
        </h2>
        <div style={{ display: "flex", gap: "10px", alignItems: "center" }}>
          {categories.length > 0 && (
            <select
              value={selectedCategory}
              onChange={(e) => setSelectedCategory(e.target.value)}
              style={{
                padding: "5px 10px",
                fontSize: "14px",
                border: "1px solid #ccc",
                borderRadius: "4px",
              }}
            >
              <option value="">全カテゴリ</option>
              {categories.map((category) => (
                <option key={category} value={category}>
                  {category}
                </option>
              ))}
            </select>
          )}
          <select
            value={pageSize}
            onChange={(e) => {
              setPageSize(Number.parseInt(e.target.value));
              setCurrentPage(1);
            }}
            style={{
              padding: "5px 10px",
              fontSize: "14px",
              border: "1px solid #ccc",
              borderRadius: "4px",
            }}
          >
            <option value={6}>6件表示</option>
            <option value={12}>12件表示</option>
            <option value={24}>24件表示</option>
            <option value={48}>48件表示</option>
          </select>
          <button type="button" className="btn btn-secondary" onClick={fetchFeeds}>
            更新
          </button>
        </div>
      </div>

      <div className="feed-grid">
        {filteredFeeds.map((feed) => (
          <div key={feed.id} className="feed-card">
            <div className="feed-card-header">
              <h3 className="feed-title">
                <Link to={`/feeds/${feed.id}`} className="feed-link">
                  {feed.title || feed.url}
                </Link>
              </h3>
              <div className="feed-url">
                <a href={feed.url} target="_blank" rel="noopener noreferrer">
                  {feed.url}
                </a>
              </div>
            </div>

            {feed.description && (
              <div className="feed-description">{feed.description}</div>
            )}

            {feed.category && (
              <div className="feed-category">
                <span className="category-badge">{feed.category}</span>
              </div>
            )}

            <div className="feed-meta">
              <div>作成日: {formatDate(feed.createdAt)}</div>
              {feed.lastUpdated && (
                <div>最終更新: {formatDate(feed.lastUpdated)}</div>
              )}
            </div>

            <div className="feed-actions">
              <Link to={`/feeds/${feed.id}`} className="btn btn-primary">
                エピソード一覧を見る
              </Link>
            </div>
          </div>
        ))}
      </div>

      {/* Pagination Controls */}
      {totalPages > 1 && (
        <div className="pagination-container">
          <div className="pagination-info">
            ページ {currentPage} / {totalPages} (全 {totalFeeds} 件)
          </div>
          <div className="pagination-controls">
            <button
              type="button"
              className="btn btn-secondary"
              onClick={() => setCurrentPage(1)}
              disabled={currentPage === 1}
            >
              最初
            </button>
            <button
              type="button"
              className="btn btn-secondary"
              onClick={() => setCurrentPage(currentPage - 1)}
              disabled={currentPage === 1}
            >
              前へ
            </button>
            
            {/* Page numbers */}
            <div className="page-numbers">
              {Array.from({ length: Math.min(5, totalPages) }, (_, i) => {
                let pageNum;
                if (totalPages <= 5) {
                  pageNum = i + 1;
                } else if (currentPage <= 3) {
                  pageNum = i + 1;
                } else if (currentPage >= totalPages - 2) {
                  pageNum = totalPages - 4 + i;
                } else {
                  pageNum = currentPage - 2 + i;
                }
                
                return (
                  <button
                    type="button"
                    key={pageNum}
                    className={`btn ${currentPage === pageNum ? 'btn-primary' : 'btn-secondary'}`}
                    onClick={() => setCurrentPage(pageNum)}
                  >
                    {pageNum}
                  </button>
                );
              })}
            </div>
            
            <button
              type="button"
              className="btn btn-secondary"
              onClick={() => setCurrentPage(currentPage + 1)}
              disabled={currentPage === totalPages}
            >
              次へ
            </button>
            <button
              type="button"
              className="btn btn-secondary"
              onClick={() => setCurrentPage(totalPages)}
              disabled={currentPage === totalPages}
            >
              最後
            </button>
          </div>
        </div>
      )}

      <style>{`
        .feed-grid {
          display: grid;
          grid-template-columns: repeat(auto-fill, minmax(300px, 1fr));
          gap: 20px;
        }

        .feed-card {
          border: 1px solid #e9ecef;
          border-radius: 8px;
          padding: 20px;
          background: white;
          box-shadow: 0 2px 4px rgba(0,0,0,0.1);
        }

        .feed-card-header {
          margin-bottom: 15px;
        }

        .feed-title {
          margin: 0 0 8px 0;
          font-size: 18px;
        }

        .feed-link {
          text-decoration: none;
          color: #007bff;
        }

        .feed-link:hover {
          text-decoration: underline;
        }

        .feed-url {
          font-size: 12px;
          color: #666;
          word-break: break-all;
        }

        .feed-url a {
          color: #666;
          text-decoration: none;
        }

        .feed-url a:hover {
          color: #007bff;
        }

        .feed-description {
          margin-bottom: 15px;
          color: #333;
          line-height: 1.5;
        }

        .feed-meta {
          margin-bottom: 15px;
          font-size: 12px;
          color: #666;
        }

        .feed-meta div {
          margin-bottom: 4px;
        }

        .feed-actions {
          text-align: right;
        }

        .feed-category {
          margin-bottom: 15px;
        }

        .category-badge {
          display: inline-block;
          background: #007bff;
          color: white;
          padding: 3px 8px;
          border-radius: 12px;
          font-size: 11px;
          font-weight: 500;
        }

        .pagination-container {
          margin-top: 30px;
          padding: 20px 0;
          border-top: 1px solid #e9ecef;
        }

        .pagination-info {
          text-align: center;
          margin-bottom: 15px;
          color: #666;
          font-size: 14px;
        }

        .pagination-controls {
          display: flex;
          justify-content: center;
          align-items: center;
          gap: 8px;
          flex-wrap: wrap;
        }

        .page-numbers {
          display: flex;
          gap: 4px;
          margin: 0 8px;
        }

        .pagination-controls button {
          min-width: 40px;
          height: 36px;
          font-size: 14px;
        }

        .pagination-controls button:disabled {
          opacity: 0.5;
          cursor: not-allowed;
        }

        @media (max-width: 768px) {
          .pagination-controls {
            flex-direction: column;
            gap: 12px;
          }
          
          .page-numbers {
            margin: 0;
          }
        }
      `}</style>
    </div>
  );
}

export default FeedList;
